from super_scad.private.PrivateImport import PrivateImport


class Import3D(PrivateImport):
    """
    Imports a file for use in the current OpenSCAD model. See
    https://en.wikibooks.org/wiki/OpenSCAD_User_Manual/Importing_Geometry#import.
    """
    pass  # Nothing to implement.

# ----------------------------------------------------------------------------------------------------------------------
